@@ -4,7 +4,6 @@
 # this file generates plain patterns (one-per-line) out of TeX source
 
 # use 'gem install unicode' if unicode is missing on your computer
-require 'unicode'
 
 require_relative 'lib/tex/hyphen/language.rb'
 # include OldLanguage::TeXLive
@@ -12,15 +11,15 @@ include TeX::Hyphen
 
 # FIXME Close files!
 # FIXME sr-cyrl?
-print 'Generating plain files for (parenthesised tags are skipped) '
+print '[1;36mGenerating[0m [0;34mplain files[0m for (parenthesised tags are skipped) '
 Language.all.sort.each do |language|
 	bcp47 = language.bcp47
 
 	if language.use_old_loader || bcp47 == 'mn-cyrl-x-lmc'
-		print '(', language.bcp47, ') '
+		print '([0;31m', language.bcp47, '[0m) '
 		next
 	else
-		print bcp47, ' '
+		print '[0;32m', bcp47, '[0m '
 	end
 
 	outfile = Proc.new do |ext|
@@ -49,16 +48,16 @@ Language.all.sort.each do |language|
 	unless language.exceptions == ""
 		file = outfile.('hyp') # This ensure a file is created, even if it may be empty
 		file.puts language.exceptions
-        end
+	end
 
-        ### FIXME Do issue #33 instead
+	### FIXME Do issue #33 instead
 	### # characters
 	### file = outfile.('chr')
 	### language.extract_characters.each do |character|
 	### 	file.puts character
 	### end
 
-        ### FIXME Do something else instead
+	### FIXME Do something else instead
 	### # comments and licence
 	### file = outfile.('lic')
 	### file.puts language.comments_and_licence
