@@ -1,6 +1,6 @@
 #!/usr/bin/env wish
 
-# Copyright 2018-2021 Siep Kroonenberg
+# Copyright 2018-2025 Siep Kroonenberg
 
 # This file is licensed under the GNU General Public License version 2
 # or any later version.
@@ -334,7 +334,7 @@ proc pre_splash {} {
   }
 
   # wallpaper for remaining widgets
-  pack [ttk::frame .bg -padding 3pt] -fill both -expand 1
+  pack [ttk::frame .bg -padding 3p] -fill both -expand 1
 
   # frame for buttons (abort button, mirrors dropdown menu)
   pack [ttk::frame .splfb] -in .bg -side bottom -fill x
@@ -389,7 +389,7 @@ proc show_log {{do_abort 0}} {
   }
 
   # wallpaper
-  pack [ttk::frame .bg -padding 3pt] -fill both -expand 1
+  pack [ttk::frame .bg -padding 3p] -fill both -expand 1
 
   # buttons at bottom
   pack [ttk::frame .bottom] -in .bg -side bottom -fill x
@@ -471,14 +471,14 @@ proc edit_name {} {
   if $::plain_unix {wm attributes .tled -type dialog}
 
   # wallpaper
-  pack [ttk::frame .tled.bg -padding 3pt] -fill both -expand 1
+  pack [ttk::frame .tled.bg -padding 3p] -fill both -expand 1
 
   # widgets
   ttk::label .tled.l -text [__ "Change name (slashes not allowed)"]
-  pack .tled.l -in .tled.bg -padx 5pt -pady 5pt
+  pack .tled.l -in .tled.bg -padx 5p -pady 5p
   ttk::entry .tled.e -width 20
   .tled.e state !disabled
-  pack .tled.e -in .tled.bg -pady 5pt
+  pack .tled.e -in .tled.bg -pady 5p
   .tled.e insert 0 [.tltd.name_l cget -text]
 
   # now frame with ok and cancel buttons
@@ -492,9 +492,9 @@ proc edit_name {} {
       end_dlg "" .tled
     }
   }
-  ppack .tled.ok_b -in .tled.buttons -side right -padx 5pt -pady 5pt
+  ppack .tled.ok_b -in .tled.buttons -side right -padx 5p -pady 5p
   ttk::button .tled.cancel_b -text [__ "Cancel"] -command {end_dlg "" .tled}
-  ppack .tled.cancel_b -in .tled.buttons -side right -padx 5pt -pady 5pt
+  ppack .tled.cancel_b -in .tled.buttons -side right -padx 5p -pady 5p
   bind .tled <Escape> {.tled.cancel_b invoke}
 
   wm protocol .tled WM_DELETE_WINDOW \
@@ -557,6 +557,7 @@ proc commit_root {} {
     set ::vars(TEXMFCONFIG) $::vars(TEXMFSYSCONFIG)
   }
   destroy .tltd
+  update_vars
 }
 
 ### main directory dialog ###
@@ -569,14 +570,14 @@ proc texdir_setup {} {
   wm title .tltd [__ "Installation root"]
 
   # wallpaper
-  pack [ttk::frame .tltd.bg -padding 3pt] -expand 1 -fill both
+  pack [ttk::frame .tltd.bg -padding 3p] -expand 1 -fill both
 
   # full path
   pack [ttk::label .tltd.path_l -font lfont -anchor center] \
-      -in .tltd.bg -pady 10pt -fill x -expand 1
+      -in .tltd.bg -pady 10p -fill x -expand 1
 
   # installation root components, gridded
-  pack [ttk::frame .tltd.fr1 -borderwidth 2pt -relief groove] \
+  pack [ttk::frame .tltd.fr1 -borderwidth 2p -relief groove] \
       -in .tltd.bg -fill x -expand 1
   grid columnconfigure .tltd.fr1 0 -weight 1
   grid columnconfigure .tltd.fr1 2 -weight 1
@@ -594,7 +595,8 @@ proc texdir_setup {} {
       -in .tltd.fr1 -row $rw -column 4
   # corresponding buttons
   incr rw
-  pgrid [ttk::button .tltd.prefix_b -text [__ "Browse..."] \
+  set prefix_text [__ "Prefix"]
+  pgrid [ttk::button .tltd.prefix_b -text "${prefix_text}... \u00B9" \
           -command {if [dirbrowser2widget .tltd.prefix_l] update_full_path}] \
       -in .tltd.fr1 -row $rw -column 0
   pgrid [ttk::button .tltd.name_b -text [__ "Change"] -command edit_name] \
@@ -603,16 +605,22 @@ proc texdir_setup {} {
       -command toggle_rel] \
       -in .tltd.fr1 -row $rw -column 4
 
+  set note_text [__ "Prefix must exist"]
+  ppack [ttk::label .tltd.notes -text "\u00B9 ${note_text}"] \
+           -in .tltd.bg -fill x -anchor w
+
   # windows: note about localized names
   if {$::tcl_platform(platform) eq "windows"} {
-    ttk::label .tltd.loc -anchor w
-    .tltd.loc configure -text \
-        [__ "Localized directory names will be replaced by their real names"]
-    ppack .tltd.loc -in .tltd.bg -fill x
+    .tltd.prefix_b configure -text "${prefix_text}... \u00B9 \u00B2"
+    set loc_text \
+      [__ "Localized directory names will be replaced by their real names"]
+    .tltd.notes configure -justify left \
+      -text "\u00B9 ${note_text}\n\u00B2 ${loc_text}"
+    ppack .tltd.notes -in .tltd.bg -fill x
   }
 
   # ok/cancel buttons
-  pack [ttk::frame .tltd.frbt] -in .tltd.bg -pady {10pt 0pt} -fill x
+  pack [ttk::frame .tltd.frbt] -in .tltd.bg -pady {10p 0p} -fill x
   ttk::button .tltd.ok_b -text [__ "Ok"] -command commit_root
   ppack .tltd.ok_b -in .tltd.frbt -side right
   ttk::button .tltd.cancel_b -text [__ "Cancel"] \
@@ -676,7 +684,7 @@ proc edit_dir {d} {
   if $::plain_unix {wm attributes .td -type dialog}
 
   # wallpaper
-  pack [ttk::frame .td.bg -padding 3pt] -fill both -expand 1
+  pack [ttk::frame .td.bg -padding 3p] -fill both -expand 1
 
   if {$d eq "TEXMFHOME"} {
     # explain tilde
@@ -881,7 +889,7 @@ proc select_binaries {} {
   wm title .tlbin [__ "Binaries"]
 
   # wallpaper
-  pack [ttk::frame .tlbin.bg -padding 3pt] -expand 1 -fill both
+  pack [ttk::frame .tlbin.bg -padding 3p] -expand 1 -fill both
 
   # ok, cancel buttons
   pack [ttk::frame .tlbin.buts] -in .tlbin.bg -side bottom -fill x
@@ -931,7 +939,7 @@ proc select_scheme {} {
   wm title .tlschm [__ "Schemes"]
 
   # wallpaper
-  pack [ttk::frame .tlschm.bg -padding 3pt] -fill both -expand 1
+  pack [ttk::frame .tlschm.bg -padding 3p] -fill both -expand 1
 
   # buttons at bottom
   pack [ttk::frame .tlschm.buts] -in .tlschm.bg -side bottom -fill x
@@ -1019,7 +1027,7 @@ proc select_collections {} {
   wm title .tlcoll [__ "Collections"]
 
   # wallpaper
-  pack [ttk::frame .tlcoll.bg -padding 3pt] -fill both -expand 1
+  pack [ttk::frame .tlcoll.bg -padding 3p] -fill both -expand 1
 
   # frame at bottom with ok and cancel buttons
   pack [ttk::frame .tlcoll.butf] -in .tlcoll.bg -side bottom -fill x
@@ -1039,14 +1047,14 @@ proc select_collections {} {
     set wgb .tlcoll.b$t
     ttk::frame $wgb
     ttk::label ${wgb}sel -text [__ "Select"]
-    ttk::button ${wgb}all -text [__ "All"] -padding 1pt -command \
+    ttk::button ${wgb}all -text [__ "All"] -padding 1p -command \
       "foreach c \[.tlcoll.$t children {}\] \{
         .tlcoll.$t set \$c mk \[mark_sym 1\]\}"
-    ttk::button ${wgb}none -text [__ "None"] -padding 1pt -command \
+    ttk::button ${wgb}none -text [__ "None"] -padding 1p -command \
       "foreach c \[.tlcoll.$t children {}\] \{
         .tlcoll.$t set \$c mk \[mark_sym 0\]\}"
     pack ${wgb}sel ${wgb}all ${wgb}none -in $wgb \
-        -side left -padx 3pt -pady 3pt
+        -side left -padx 3p -pady 3p
 
     # trees with collections and markers, lang and other separately
     set wgt ".tlcoll.$t"
@@ -1205,7 +1213,7 @@ if {$::tcl_platform(platform) ne "windows"} {
     create_dlg .edsyms .
     wm title .edsyms [__ "Symlinks"]
 
-    pack [ttk::frame .edsyms.bg -padding 3pt] -expand 1 -fill both
+    pack [ttk::frame .edsyms.bg -padding 3p] -expand 1 -fill both
     set rw -1
 
     pack [ttk::frame .edsyms.fr0] -in .edsyms.bg -expand 1 -fill both
@@ -1301,6 +1309,15 @@ proc abort_menu {} {
   # i.e. anything but advanced, alltrees or startinst
 }
 
+proc maybe_install {} {
+  if {($::vars(free_size)!=-1) && \
+          ($::vars(total_size) >= ($::vars(free_size)-100))} {
+    tk_messageBox -icon error -message [__ "Not enough room"]
+  } else {
+    set ::menu_ans "startinst"
+  }
+}
+
 proc run_menu {} {
   #if [info exists ::env(dbgui)] {
   #  puts "\ndbgui: run_menu: advanced is now $::advanced"
@@ -1328,7 +1345,7 @@ proc run_menu {} {
   menu .mn
   . configure -menu .mn
   if $::plain_unix {
-    .mn configure -borderwidth 1pt
+    .mn configure -borderwidth 1p
     .mn configure -background $::default_bg
   }
   .mn add command -command abort_menu -label [__ "Abort"]
@@ -1370,22 +1387,22 @@ proc run_menu {} {
   }
 
   # wallpaper, for a uniform background
-  pack [ttk::frame .bg -padding 3pt] -fill both -expand 1
+  pack [ttk::frame .bg -padding 3p] -fill both -expand 1
 
   # title
   ttk::label .title -text [__ "TeX Live %s Installer" $::release_year] \
       -font titlefont
-  pack .title -pady {10pt 1pt} -in .bg
+  pack .title -pady {10p 1p} -in .bg
   pack [ttk::label .svn -text "r. $::svn"] -in .bg
 
   pack [ttk::separator .seph0 -orient horizontal] \
-      -in .bg -pady 3pt -fill x
+      -in .bg -pady 3p -fill x
 
   # frame at bottom with install/quit buttons
   pack [ttk::frame .final] \
-      -in .bg -side bottom -pady {5pt 2pt} -fill x
-  ppack [ttk::button .install -text [__ "Install"] -command {
-    set ::menu_ans "startinst"}] -in .final -side right
+      -in .bg -side bottom -pady {5p 2p} -fill x
+  ppack [ttk::button .install -text [__ "Install"] -command maybe_install] \
+    -in .final -side right
   ppack [ttk::button .quit -text [__ "Quit"] -command {
     set ::out_log {}
     set ::menu_ans "no_inst"}] -in .final -side right
@@ -1397,7 +1414,7 @@ proc run_menu {} {
     }] -in .final -side left
   }
   pack [ttk::separator .seph1 -orient horizontal] \
-      -in .bg -side bottom -pady 3pt -fill x
+      -in .bg -side bottom -pady 3p -fill x
 
   # directories, selections
   # advanced and basic have different frame setups
@@ -1570,7 +1587,7 @@ proc run_menu {} {
         -in .selsf -row $rw -column 2 -sticky e
   }
 
-  # total size
+  # total size and available space
   # curf: current frame
   set curf [expr {$::advanced ? ".selsf" : ".dirf"}]
   incr rw
@@ -1578,6 +1595,13 @@ proc run_menu {} {
   ttk::label .size_req -textvariable ::vars(total_size)
   pgrid .lsize -in $curf -row $rw -column 0 -sticky w
   pgrid .size_req -in $curf -row $rw -column 1 -sticky w
+  if {$::vars(free_size) != -1} {
+    incr rw
+    ttk::label .lavail -text [__ "Disk space available (in MB):"]
+    ttk::label .avail -textvariable ::vars(free_size)
+    pgrid .lavail -in $curf -row $rw -column 0 -sticky w
+    pgrid .avail -in $curf -row $rw -column 1 -sticky w
+  }
 
   ########################################################
   # right side: options
@@ -1586,7 +1610,7 @@ proc run_menu {} {
   if $::advanced {
 
     pack [ttk::separator .sepv -orient vertical] \
-        -in .bg -side left -padx 3pt -fill y
+        -in .bg -side left -padx 3p -fill y
     pack [ttk::frame .options] -in .bg -side right -fill both -expand 1
 
     set curf .options
